"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../../shared/utils");
var md5_1 = __importDefault(require("md5"));
var StorageAccountResource = /** @class */ (function () {
    function StorageAccountResource() {
    }
    StorageAccountResource.getResourceName = function (config) {
        return config.provider.storageAccount && config.provider.storageAccount.name
            ? config.provider.storageAccount.name
            : StorageAccountResource.getDefaultStorageAccountName(config);
    };
    StorageAccountResource.prototype.getTemplate = function () {
        return {
            "$schema": "https://schema.management.azure.com/schemas/2015-01-01/deploymentTemplate.json#",
            "contentVersion": "1.0.0.0",
            "parameters": {
                "storageAccountName": {
                    "defaultValue": "",
                    "type": "String"
                },
                "location": {
                    "defaultValue": "",
                    "type": "String"
                },
                "storageAccountSkuName": {
                    "defaultValue": "Standard_LRS",
                    "type": "String"
                },
                "storageAccoutSkuTier": {
                    "defaultValue": "Standard",
                    "type": "String"
                }
            },
            "variables": {},
            "resources": [
                {
                    "apiVersion": "2018-07-01",
                    "name": "[parameters('storageAccountName')]",
                    "type": "Microsoft.Storage/storageAccounts",
                    "location": "[parameters('location')]",
                    "kind": "Storage",
                    "properties": {
                        "accountType": "[parameters('storageAccountSkuName')]"
                    },
                    "sku": {
                        "name": "[parameters('storageAccountSkuName')]",
                        "tier": "[parameters('storageAccoutSkuTier')]"
                    }
                }
            ]
        };
    };
    StorageAccountResource.prototype.getParameters = function (config) {
        var resourceConfig = __assign({ sku: {} }, config.provider.storageAccount);
        return {
            storageAccountName: StorageAccountResource.getResourceName(config),
            storageAccountSkuName: resourceConfig.sku.name,
            storageAccoutSkuTier: resourceConfig.sku.tier,
        };
    };
    /**
     * Gets a default storage account name.
     * Storage account names can have at most 24 characters and can have only alpha-numerics
     * @param config Serverless Azure Config
     */
    StorageAccountResource.getDefaultStorageAccountName = function (config) {
        var maxAccountNameLength = 24;
        var nameHash = md5_1.default(config.service);
        var replacer = /\W+/g;
        var safePrefix = config.provider.prefix.replace(replacer, "");
        var safeRegion = utils_1.Utils.createShortAzureRegionName(config.provider.region);
        var safeStage = utils_1.Utils.createShortStageName(config.provider.stage);
        var safeNameHash = nameHash.substr(0, 6);
        var remaining = maxAccountNameLength - (safePrefix.length + safeRegion.length + safeStage.length + safeNameHash.length);
        // Dynamically adjust the substring based on space needed
        if (remaining < 0) {
            var partLength = Math.floor(Math.abs(remaining) / 3);
            safePrefix = safePrefix.substr(0, partLength);
            safeStage = safeStage.substr(0, partLength);
            safeNameHash = safeNameHash.substr(0, partLength);
        }
        return [safePrefix, safeRegion, safeStage, safeNameHash]
            .join("")
            .toLocaleLowerCase();
    };
    return StorageAccountResource;
}());
exports.StorageAccountResource = StorageAccountResource;
