"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = require("path");
var request_1 = __importDefault(require("request"));
var config_1 = __importDefault(require("../config"));
var functionAppName;
var functionsAdminKey;
var invocationId;
var AzureProvider = /** @class */ (function () {
    function AzureProvider(serverless) {
        this.serverless = serverless;
        this.serverless.setProvider(config_1.default.providerName, this);
    }
    AzureProvider.getProviderName = function () {
        return config_1.default.providerName;
    };
    AzureProvider.prototype.getAdminKey = function () {
        var options = {
            url: "https://" + functionAppName + config_1.default.scmDomain + config_1.default.masterKeyApiPath,
            json: true,
            headers: {
                Authorization: config_1.default.bearer + this.credentials.tokenCache._entries[0].accessToken
            }
        };
        return new Promise(function (resolve, reject) {
            request_1.default(options, function (err, response, body) {
                if (err)
                    return reject(err);
                if (response.statusCode !== 200)
                    return reject(body);
                functionsAdminKey = body.masterKey;
                resolve(body.masterKey);
            });
        });
    };
    AzureProvider.prototype.pingHostStatus = function (functionName) {
        var _this = this;
        var requestUrl = "https://" + functionAppName + config_1.default.functionAppDomain + "/admin/functions/" + functionName + "/status";
        var options = {
            host: functionAppName + config_1.default.functionAppDomain,
            method: "get",
            url: requestUrl,
            json: true,
            headers: {
                "x-functions-key": functionsAdminKey,
                Accept: "application/json,*/*"
            }
        };
        return new Promise(function (resolve, reject) {
            _this.serverless.cli.log("Pinging host status...");
            request_1.default(options, function (err, res, body) {
                if (err)
                    return reject(err);
                if (body && body.Error)
                    return reject(body.Error);
                if (res.statusCode !== 200) {
                    return body && body.Error ? reject(body.Error) : reject(body);
                }
                resolve(res);
            });
        });
    };
    AzureProvider.prototype.getLogsStream = function (functionName) {
        var _this = this;
        var logOptions = {
            url: "https://" + functionAppName + config_1.default.scmDomain + config_1.default.logStreamApiPath + functionName,
            headers: {
                Authorization: config_1.default.bearer + this.credentials.tokenCache._entries[0].accessToken,
                Accept: "*/*"
            }
        };
        request_1.default
            .get(logOptions)
            .on("error", function () {
            console.error("Disconnected from log streaming.");
        })
            .on("end", function () { return _this.getLogsStream(functionName); })
            .pipe(process.stdout);
    };
    AzureProvider.prototype.getInvocationId = function (functionName) {
        var options = {
            url: "https://" + functionAppName + config_1.default.scmDomain + (config_1.default.logInvocationsApiPath + functionAppName) + "-" + functionName + "/invocations?limit=5",
            method: "GET",
            json: true,
            headers: {
                Authorization: config_1.default.bearer + this.credentials.tokenCache._entries[0].accessToken
            }
        };
        return new Promise(function (resolve, reject) {
            request_1.default(options, function (err, response, body) {
                if (err)
                    return reject(err);
                if (response.statusCode !== 200)
                    return reject(body);
                invocationId = body.entries[0].id;
                resolve(body.entries[0].id);
            });
        });
    };
    AzureProvider.prototype.getLogsForInvocationId = function () {
        this.serverless.cli.log("Logs for InvocationId: " + invocationId);
        var options = {
            url: "https://" + functionAppName + config_1.default.scmDomain + config_1.default.logOutputApiPath + invocationId,
            method: "GET",
            json: true,
            headers: {
                Authorization: config_1.default.bearer + this.credentials.tokenCache._entries[0].accessToken
            }
        };
        return new Promise(function (resolve, reject) {
            request_1.default(options, function (err, response, body) {
                if (err)
                    return reject(err);
                if (response.statusCode !== 200)
                    return reject(body);
                resolve(body);
            });
        });
    };
    AzureProvider.prototype.invoke = function (functionName, eventType, eventData) {
        var _this = this;
        if (eventType === "http") {
            var queryString_1 = "";
            if (eventData) {
                if (typeof eventData === "string") {
                    try {
                        eventData = JSON.parse(eventData);
                    }
                    catch (error) {
                        return Promise.reject("The specified input data isn't a valid JSON string. " +
                            "Please correct it and try invoking the function again.");
                    }
                }
                queryString_1 = Object.keys(eventData)
                    .map(function (key) { return key + "=" + eventData[key]; })
                    .join("&");
            }
            return new Promise(function (resolve, reject) {
                var options = {
                    headers: {
                        "x-functions-key": functionsAdminKey
                    },
                    url: "http://" + functionAppName + config_1.default.functionAppDomain + (config_1.default.functionAppApiPath + functionName) + "?" + queryString_1,
                    method: "GET",
                    json: true,
                };
                _this.serverless.cli.log("Invoking function \"" + functionName + "\"");
                request_1.default(options, function (err, response, body) {
                    if (err)
                        return reject(err);
                    if (response.statusCode !== 200)
                        return reject(body);
                    console.log(body);
                    resolve(body);
                });
            });
        }
        var requestUrl = "https://" + functionAppName + config_1.default.functionAppDomain + config_1.default.functionsAdminApiPath + functionName;
        var options = {
            host: config_1.default.functionAppDomain,
            method: "post",
            body: eventData,
            url: requestUrl,
            json: true,
            headers: {
                "x-functions-key": functionsAdminKey,
                Accept: "application/json,*/*"
            }
        };
        return new Promise(function (resolve, reject) {
            request_1.default(options, function (err, res) {
                if (err) {
                    reject(err);
                }
                _this.serverless.cli.log("Invoked function at: " + requestUrl + ". \nResponse statuscode: " + res.statusCode);
                resolve(res);
            });
        });
    };
    AzureProvider.prototype.uploadPackageJson = function () {
        var packageJsonFilePath = path_1.join(this.serverless.config.servicePath, "package.json");
        this.serverless.cli.log("Uploading package.json...");
        var requestUrl = "https://" + functionAppName + config_1.default.scmDomain + config_1.default.scmVfsPath + "package.json";
        var options = {
            host: functionAppName + config_1.default.scmDomain,
            method: "put",
            url: requestUrl,
            json: true,
            headers: {
                Authorization: config_1.default.bearer + this.credentials.tokenCache._entries[0].accessToken,
                Accept: "*/*"
            }
        };
        return new Promise(function (resolve, reject) {
            if (fs_1.default.existsSync(packageJsonFilePath)) {
                fs_1.default.createReadStream(packageJsonFilePath)
                    .pipe(request_1.default.put(options, function (err) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve("Package.json file uploaded");
                    }
                }));
            }
            else {
                resolve("Package.json file does not exist");
            }
        });
    };
    return AzureProvider;
}());
exports.default = AzureProvider;
