"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var mockFactory_1 = require("../test/mockFactory");
var armService_1 = require("./armService");
var armTemplates_1 = require("../models/armTemplates");
var mock_fs_1 = __importDefault(require("mock-fs"));
var jsonpath_1 = __importDefault(require("jsonpath"));
var arm_resources_1 = require("@azure/arm-resources");
describe("Arm Service", function () {
    var sls;
    var service;
    var options;
    function createService() {
        return new armService_1.ArmService(sls, options);
    }
    beforeEach(function () {
        sls = mockFactory_1.MockFactory.createTestServerless();
        sls.service.provider["prefix"] = "myapp";
        sls.service.provider.region = "westus";
        sls.service.provider.stage = "dev";
        sls.variables = __assign({}, sls.variables, { azureCredentials: mockFactory_1.MockFactory.createTestAzureCredentials(), subscriptionId: "ABC123" });
        service = createService();
    });
    afterEach(function () {
        mock_fs_1.default.restore();
    });
    describe("Creating Templates", function () {
        it("Creates an ARM template from a specified file", function () { return __awaiter(_this, void 0, void 0, function () {
            var armTemplateConfig, testTemplate, deployment;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        armTemplateConfig = {
                            file: "armTemplates/custom-template.json",
                            parameters: {
                                param1: "1",
                                param2: "2",
                            },
                        };
                        testTemplate = mockFactory_1.MockFactory.createTestArmTemplate();
                        mock_fs_1.default({
                            "armTemplates": {
                                "custom-template.json": JSON.stringify(testTemplate),
                            },
                        });
                        sls.service.provider["armTemplate"] = armTemplateConfig;
                        return [4 /*yield*/, service.createDeploymentFromConfig(sls.service.provider["armTemplate"])];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment).not.toBeNull();
                        expect(deployment.template.parameters).toEqual(testTemplate.parameters);
                        expect(deployment.template.resources).toEqual(testTemplate.resources);
                        expect(deployment.parameters).toEqual(armTemplateConfig.parameters);
                        return [2 /*return*/];
                }
            });
        }); });
        it("Creates a custom ARM template from well-known type", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, service.createDeploymentFromType("premium")];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment).not.toBeNull();
                        expect(Object.keys(deployment.parameters).length).toBeGreaterThan(0);
                        expect(deployment.template.resources.length).toBeGreaterThan(0);
                        return [2 /*return*/];
                }
            });
        }); });
        it("Creates a custom ARM template (with APIM support) from well-known type", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls.service.provider["apim"] = mockFactory_1.MockFactory.createTestApimConfig();
                        return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.Premium)];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment).not.toBeNull();
                        expect(Object.keys(deployment.parameters).length).toBeGreaterThan(0);
                        expect(deployment.template.resources.length).toBeGreaterThan(0);
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.ApiManagement/service"; })).not.toBeNull();
                        return [2 /*return*/];
                }
            });
        }); });
        it("throws error when specified type is not found", function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, expect(service.createDeploymentFromType("not-found")).rejects.not.toBeNull()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it("Premium template includes correct resources", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment, functionApp;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.Premium)];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment.template.parameters.appServicePlanSkuTier.defaultValue).toEqual("ElasticPremium");
                        expect(deployment.template.parameters.appServicePlanSkuName.defaultValue).toEqual("EP1");
                        // Should not contain
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/hostingEnvironments"; })).toBeUndefined();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Network/virtualNetworks"; })).toBeUndefined();
                        // Should contain
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/serverfarms"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/sites"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Storage/storageAccounts"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "microsoft.insights/components"; })).not.toBeNull();
                        functionApp = deployment.template.resources.find(function (res) { return res.type === "Microsoft.Web/sites"; });
                        expect(functionApp.dependsOn).toContain("[concat('Microsoft.Web/serverfarms/', parameters('appServicePlanName'))]");
                        expect(functionApp.properties.serverFarmId).toEqual("[resourceId('Microsoft.Web/serverfarms', parameters('appServicePlanName'))]");
                        return [2 /*return*/];
                }
            });
        }); });
        it("ASE template includes correct resources", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment, appServicePlan, functionApp;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.AppServiceEnvironment)];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment.template.parameters.appServicePlanSkuTier.defaultValue).toEqual("Isolated");
                        expect(deployment.template.parameters.appServicePlanSkuName.defaultValue).toEqual("I1");
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/hostingEnvironments"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Network/virtualNetworks"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/serverfarms"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/sites"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Storage/storageAccounts"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "microsoft.insights/components"; })).not.toBeNull();
                        appServicePlan = deployment.template.resources.find(function (res) { return res.type === "Microsoft.Web/serverfarms"; });
                        expect(appServicePlan.dependsOn).toContain("[resourceId('Microsoft.Web/hostingEnvironments', parameters('hostingEnvironmentName'))]");
                        expect(appServicePlan.properties.hostingEnvironmentProfile.id).toEqual("[resourceId('Microsoft.Web/hostingEnvironments', parameters('hostingEnvironmentName'))]");
                        functionApp = deployment.template.resources.find(function (res) { return res.type === "Microsoft.Web/sites"; });
                        expect(functionApp.dependsOn).toContain("[concat('Microsoft.Web/serverfarms/', parameters('appServicePlanName'))]");
                        expect(functionApp.properties.serverFarmId).toEqual("[resourceId('Microsoft.Web/serverfarms', parameters('appServicePlanName'))]");
                        expect(functionApp.properties.hostingEnvironmentProfile.id).toEqual("[resourceId('Microsoft.Web/hostingEnvironments', parameters('hostingEnvironmentName'))]");
                        return [2 /*return*/];
                }
            });
        }); });
        it("Consumption template includes correct resources", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.Consumption)];
                    case 1:
                        deployment = _a.sent();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/hostingEnvironments"; })).toBeUndefined();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Network/virtualNetworks"; })).toBeUndefined();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/serverfarms"; })).toBeUndefined();
                        // Should contain
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Web/sites"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "Microsoft.Storage/storageAccounts"; })).not.toBeNull();
                        expect(deployment.template.resources.find(function (resource) { return resource.type === "microsoft.insights/components"; })).not.toBeNull();
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe("Deploying Templates", function () {
        beforeEach(function () {
            arm_resources_1.Deployments.prototype.createOrUpdate = jest.fn(function () { return Promise.resolve(null); });
        });
        it("Appends environment variables into app settings of ARM template", function () { return __awaiter(_this, void 0, void 0, function () {
            var environmentConfig, deployment, appSettings;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        environmentConfig = {
                            PARAM_1: "1",
                            PARAM_2: "2",
                            PARAM_3: "3",
                        };
                        sls.service.provider["environment"] = environmentConfig;
                        return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.Consumption)];
                    case 1:
                        deployment = _a.sent();
                        return [4 /*yield*/, service.deployTemplate(deployment)];
                    case 2:
                        _a.sent();
                        appSettings = jsonpath_1.default.query(deployment.template, "$.resources[?(@.kind==\"functionapp\")].properties.siteConfig.appSettings[*]");
                        expect(appSettings.find(function (setting) { return setting.name === "PARAM_1"; })).toEqual({ name: "PARAM_1", value: environmentConfig.PARAM_1 });
                        expect(appSettings.find(function (setting) { return setting.name === "PARAM_2"; })).toEqual({ name: "PARAM_2", value: environmentConfig.PARAM_2 });
                        expect(appSettings.find(function (setting) { return setting.name === "PARAM_3"; })).toEqual({ name: "PARAM_3", value: environmentConfig.PARAM_3 });
                        return [2 /*return*/];
                }
            });
        }); });
        it("Deploys ARM template via resources REST API", function () { return __awaiter(_this, void 0, void 0, function () {
            var deployment, deploymentParameters, expectedResourceGroup, expectedDeploymentName, expectedDeploymentNameRegex, expectedDeployment, call;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, service.createDeploymentFromType(armTemplates_1.ArmTemplateType.Consumption)];
                    case 1:
                        deployment = _a.sent();
                        deploymentParameters = {};
                        Object.keys(deployment.parameters).forEach(function (key) {
                            var parameterValue = deployment.parameters[key];
                            if (parameterValue) {
                                deploymentParameters[key] = { value: deployment.parameters[key] };
                            }
                        });
                        return [4 /*yield*/, service.deployTemplate(deployment)];
                    case 2:
                        _a.sent();
                        expectedResourceGroup = sls.service.provider["resourceGroup"];
                        expectedDeploymentName = sls.service.provider["deploymentName"] || this.resourceGroup + "-deployment";
                        expectedDeploymentNameRegex = new RegExp(expectedDeploymentName + "-t([0-9]+)");
                        expectedDeployment = {
                            properties: {
                                mode: "Incremental",
                                template: deployment.template,
                                parameters: deploymentParameters,
                            },
                        };
                        call = arm_resources_1.Deployments.prototype.createOrUpdate.mock.calls[0];
                        expect(call[0]).toEqual(expectedResourceGroup);
                        expect(call[1]).toMatch(expectedDeploymentNameRegex);
                        expect(call[2]).toEqual(expectedDeployment);
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
