"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var mock_fs_1 = __importDefault(require("mock-fs"));
var mockFactory_1 = require("../test/mockFactory");
var azureBlobStorageService_1 = require("./azureBlobStorageService");
jest.mock("@azure/storage-blob");
jest.genMockFromModule("@azure/storage-blob");
var storage_blob_1 = require("@azure/storage-blob");
jest.mock("@azure/arm-storage");
jest.genMockFromModule("@azure/arm-storage");
var arm_storage_1 = require("@azure/arm-storage");
jest.mock("./loginService");
var loginService_1 = require("./loginService");
var storageAccount_1 = require("../armTemplates/resources/storageAccount");
jest.mock("fs");
var fs_1 = __importDefault(require("fs"));
describe("Azure Blob Storage Service", function () {
    var filePath = "deployments/deployment.zip";
    var fileName = "deployment.zip";
    var fileContents = "contents";
    var containerName = "DEPLOYMENTS";
    var containers = mockFactory_1.MockFactory.createTestAzureContainers();
    var sls = mockFactory_1.MockFactory.createTestServerless();
    var accountName = storageAccount_1.StorageAccountResource.getResourceName(sls.service);
    var options = mockFactory_1.MockFactory.createTestServerlessOptions();
    var blobContent = "testContent";
    var blockBlobUrl = mockFactory_1.MockFactory.createTestBlockBlobUrl(containerName, filePath, blobContent);
    var service;
    var token = "myToken";
    var keyValue = "keyValue";
    beforeAll(function () {
        storage_blob_1.SharedKeyCredential.mockImplementation();
        storage_blob_1.TokenCredential.mockImplementation();
        arm_storage_1.StorageAccounts.prototype.listKeys = jest.fn(function () {
            return {
                keys: [
                    {
                        value: keyValue
                    }
                ]
            };
        });
        storage_blob_1.BlockBlobURL.fromContainerURL = jest.fn(function () { return blockBlobUrl; });
        loginService_1.AzureLoginService.login = jest.fn(function () { return Promise.resolve({
            credentials: {
                getToken: jest.fn(function () {
                    return {
                        accessToken: token
                    };
                })
            }
        }); });
    });
    beforeAll(function () {
        mock_fs_1.default({
            "deployments/deployment.zip": fileContents
        });
    });
    afterAll(function () {
        mock_fs_1.default.restore();
    });
    beforeEach(function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    service = new azureBlobStorageService_1.AzureBlobStorageService(sls, options);
                    return [4 /*yield*/, service.initialize()];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
    it("should initialize authentication", function () { return __awaiter(_this, void 0, void 0, function () {
        var tokenService;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    // Note: initialize called in beforeEach
                    expect(storage_blob_1.SharedKeyCredential).toBeCalledWith(accountName, keyValue);
                    expect(arm_storage_1.StorageManagementClientContext).toBeCalled();
                    expect(arm_storage_1.StorageAccounts).toBeCalled();
                    tokenService = new azureBlobStorageService_1.AzureBlobStorageService(sls, options, azureBlobStorageService_1.AzureStorageAuthType.Token);
                    return [4 /*yield*/, tokenService.initialize()];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.TokenCredential).toBeCalled();
                    return [2 /*return*/];
            }
        });
    }); });
    it("should initialize authentication", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, service.initialize()];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.TokenCredential).toBeCalledWith(token);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should upload a file", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    storage_blob_1.uploadFileToBlockBlob.prototype = jest.fn(function () { return Promise.resolve(); });
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function (serviceUrl, containerName) { return containerName; });
                    return [4 /*yield*/, service.uploadFile(filePath, containerName)];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.uploadFileToBlockBlob).toBeCalledWith(storage_blob_1.Aborter.none, filePath, blockBlobUrl);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should delete a file", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function (serviceUrl, containerName) { return containerName; });
                    return [4 /*yield*/, service.deleteFile(containerName, fileName)];
                case 1:
                    _a.sent();
                    expect(blockBlobUrl.delete).toBeCalledWith(storage_blob_1.Aborter.none);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should list files of container", function () { return __awaiter(_this, void 0, void 0, function () {
        var blobs, files, otherFiles;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    blobs = mockFactory_1.MockFactory.createTestAzureBlobItems();
                    storage_blob_1.ContainerURL.prototype.listBlobFlatSegment = jest.fn(function () { return Promise.resolve(blobs); });
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function () { return new storage_blob_1.ContainerURL(null, null); });
                    return [4 /*yield*/, service.listFiles(containerName)];
                case 1:
                    files = _a.sent();
                    expect(files.length).toEqual(5);
                    return [4 /*yield*/, service.listFiles(containerName, "jpeg")];
                case 2:
                    otherFiles = _a.sent();
                    expect(otherFiles.length).toEqual(0);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should list containers", function () { return __awaiter(_this, void 0, void 0, function () {
        var containerList;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    storage_blob_1.ServiceURL.prototype.listContainersSegment = jest.fn(function () { return Promise.resolve(containers); });
                    return [4 /*yield*/, service.listContainers()];
                case 1:
                    containerList = _a.sent();
                    expect(containerList).toEqual(containers.containerItems.map(function (container) { return container.name; }));
                    return [2 /*return*/];
            }
        });
    }); });
    it("should create a container", function () { return __awaiter(_this, void 0, void 0, function () {
        var newContainerName;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function () { return new storage_blob_1.ContainerURL(null, null); });
                    storage_blob_1.ContainerURL.prototype.create = jest.fn(function () { return Promise.resolve({ statusCode: 201 }); });
                    newContainerName = "newContainer";
                    return [4 /*yield*/, service.createContainerIfNotExists(newContainerName)];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.ContainerURL.fromServiceURL).toBeCalledWith(expect.anything(), newContainerName);
                    expect(storage_blob_1.ContainerURL.prototype.create).toBeCalledWith(storage_blob_1.Aborter.none);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should not create a container if it exists already", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function () { return new storage_blob_1.ContainerURL(null, null); });
                    storage_blob_1.ContainerURL.prototype.create = jest.fn(function () { return Promise.resolve({ statusCode: 201 }); });
                    return [4 /*yield*/, service.createContainerIfNotExists("container1")];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.ContainerURL.fromServiceURL).not.toBeCalled();
                    expect(storage_blob_1.ContainerURL.prototype.create).not.toBeCalled;
                    return [2 /*return*/];
            }
        });
    }); });
    it("should delete a container", function () { return __awaiter(_this, void 0, void 0, function () {
        var containerToDelete;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    containerToDelete = "delete container";
                    storage_blob_1.ContainerURL.fromServiceURL = jest.fn(function () { return new storage_blob_1.ContainerURL(null, null); });
                    storage_blob_1.ContainerURL.prototype.delete = jest.fn(function () { return Promise.resolve({ statusCode: 204 }); });
                    return [4 /*yield*/, service.deleteContainer(containerToDelete)];
                case 1:
                    _a.sent();
                    expect(storage_blob_1.ContainerURL.fromServiceURL).toBeCalledWith(expect.anything(), containerToDelete);
                    expect(storage_blob_1.ContainerURL.prototype.delete).toBeCalledWith(storage_blob_1.Aborter.none);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should download a binary file", function () { return __awaiter(_this, void 0, void 0, function () {
        var targetPath, buffer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    targetPath = "";
                    return [4 /*yield*/, service.downloadBinary(containerName, fileName, targetPath)];
                case 1:
                    _a.sent();
                    buffer = Buffer.alloc(blobContent.length);
                    expect(storage_blob_1.downloadBlobToBuffer).toBeCalledWith(storage_blob_1.Aborter.timeout(30 * 60 * 1000), buffer, blockBlobUrl, 0, undefined, {
                        blockSize: 4 * 1024 * 1024,
                        parallelism: 20,
                    });
                    expect(fs_1.default.writeFileSync).toBeCalledWith(targetPath, buffer, "binary");
                    return [2 /*return*/];
            }
        });
    }); });
    it("should generate a SAS url", function () { return __awaiter(_this, void 0, void 0, function () {
        var sasToken, url;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sasToken = "myToken";
                    storage_blob_1.BlobSASPermissions.parse = jest.fn(function () {
                        return {
                            toString: jest.fn()
                        };
                    });
                    storage_blob_1.generateBlobSASQueryParameters.mockReturnValue(token);
                    return [4 /*yield*/, service.generateBlobSasTokenUrl(containerName, fileName)];
                case 1:
                    url = _a.sent();
                    expect(storage_blob_1.generateBlobSASQueryParameters).toBeCalled();
                    expect(url).toEqual(blockBlobUrl.url + "?" + sasToken);
                    return [2 /*return*/];
            }
        });
    }); });
    it("should throw an error when trying to get a SAS Token with Token Auth", function () { return __awaiter(_this, void 0, void 0, function () {
        var newService;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    newService = new azureBlobStorageService_1.AzureBlobStorageService(sls, options, azureBlobStorageService_1.AzureStorageAuthType.Token);
                    return [4 /*yield*/, newService.initialize()];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, expect(newService.generateBlobSasTokenUrl(containerName, fileName)).rejects.not.toBeNull()];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
});
