"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../shared/utils");
var mockFactory_1 = require("../test/mockFactory");
var resourceService_1 = require("./resourceService");
jest.mock("@azure/arm-resources");
var arm_resources_1 = require("@azure/arm-resources");
describe("Resource Service", function () {
    var deployments;
    var template = "myTemplate";
    beforeEach(function () {
        deployments = mockFactory_1.MockFactory.createTestDeployments(5, true);
        arm_resources_1.ResourceManagementClient.prototype.resourceGroups = {
            createOrUpdate: jest.fn(),
            deleteMethod: jest.fn(),
        };
        arm_resources_1.ResourceManagementClient.prototype.deployments = {
            deleteMethod: jest.fn(),
            listByResourceGroup: jest.fn(function () { return Promise.resolve(deployments); }),
            exportTemplate: jest.fn(function () { return Promise.resolve(template); }),
        };
    });
    it("throws error with empty credentials", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        delete sls.variables["azureCredentials"];
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        expect(function () { return new resourceService_1.ResourceService(sls, options); }).toThrowError("Azure Credentials has not been set in ResourceService");
    });
    it("initializes a resource service", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        expect(function () { return new resourceService_1.ResourceService(sls, options); }).not.toThrowError();
    });
    it("deploys a resource group", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        var location = "West Us";
        var expectedLocation = utils_1.Utils.getNormalizedRegionName(location);
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.service.provider.region = location;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deployResourceGroup();
        expect(arm_resources_1.ResourceManagementClient.prototype.resourceGroups.createOrUpdate)
            .toBeCalledWith(resourceGroup, { location: expectedLocation });
    });
    it("deletes a deployment", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        var deploymentName = "myDeployment";
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.service.provider["deploymentName"] = deploymentName;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deleteDeployment();
        var call = arm_resources_1.ResourceManagementClient.prototype.deployments.deleteMethod.mock.calls[0];
        expect(call[0]).toEqual(resourceGroup);
        var expectedDeploymentNameRegex = new RegExp(deploymentName + "-t([0-9]+)");
        expect(call[1]).toMatch(expectedDeploymentNameRegex);
    });
    it("deletes a resource group", function () {
        var sls = mockFactory_1.MockFactory.createTestServerless();
        var resourceGroup = "myResourceGroup";
        sls.service.provider["resourceGroup"] = resourceGroup;
        sls.variables["azureCredentials"] = "fake credentials";
        var options = mockFactory_1.MockFactory.createTestServerlessOptions();
        var service = new resourceService_1.ResourceService(sls, options);
        service.deleteResourceGroup();
        expect(arm_resources_1.ResourceManagementClient.prototype.resourceGroups.deleteMethod)
            .toBeCalledWith(resourceGroup);
    });
    it("gets deployments", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, resourceGroup, options, service, deps;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    resourceGroup = "myResourceGroup";
                    sls.service.provider["resourceGroup"] = resourceGroup;
                    sls.variables["azureCredentials"] = "fake credentials";
                    options = mockFactory_1.MockFactory.createTestServerlessOptions();
                    service = new resourceService_1.ResourceService(sls, options);
                    return [4 /*yield*/, service.getDeployments()];
                case 1:
                    deps = _a.sent();
                    expect(deps).toEqual(deployments);
                    return [2 /*return*/];
            }
        });
    }); });
    it("lists deployments as string with timestamps", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, resourceGroup, options, service, deploymentString, expectedDeploymentString, originalTimestamp, i, deployments_1, deployments_1_1, dep, timestamp;
        var e_1, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    resourceGroup = "myResourceGroup";
                    sls.service.provider["resourceGroup"] = resourceGroup;
                    sls.variables["azureCredentials"] = "fake credentials";
                    options = mockFactory_1.MockFactory.createTestServerlessOptions();
                    service = new resourceService_1.ResourceService(sls, options);
                    return [4 /*yield*/, service.listDeployments()];
                case 1:
                    deploymentString = _b.sent();
                    expectedDeploymentString = "\n\nDeployments";
                    originalTimestamp = +mockFactory_1.MockFactory.createTestTimestamp();
                    i = 0;
                    try {
                        for (deployments_1 = __values(deployments), deployments_1_1 = deployments_1.next(); !deployments_1_1.done; deployments_1_1 = deployments_1.next()) {
                            dep = deployments_1_1.value;
                            timestamp = originalTimestamp + i;
                            expectedDeploymentString += "\n-----------\n";
                            expectedDeploymentString += "Name: " + dep.name + "\n";
                            expectedDeploymentString += "Timestamp: " + timestamp + "\n";
                            expectedDeploymentString += "Datetime: " + new Date(timestamp).toISOString() + "\n";
                            i++;
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (deployments_1_1 && !deployments_1_1.done && (_a = deployments_1.return)) _a.call(deployments_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    expectedDeploymentString += "-----------\n";
                    expect(deploymentString).toEqual(expectedDeploymentString);
                    return [2 /*return*/];
            }
        });
    }); });
    it("lists deployments as string without timestamps", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, resourceGroup, options, service, deploymentString, expectedDeploymentString, deployments_2, deployments_2_1, dep;
        var e_2, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    deployments = mockFactory_1.MockFactory.createTestDeployments();
                    arm_resources_1.ResourceManagementClient.prototype.deployments = {
                        listByResourceGroup: jest.fn(function () { return Promise.resolve(deployments); }),
                    };
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    resourceGroup = "myResourceGroup";
                    sls.service.provider["resourceGroup"] = resourceGroup;
                    sls.variables["azureCredentials"] = "fake credentials";
                    options = mockFactory_1.MockFactory.createTestServerlessOptions();
                    service = new resourceService_1.ResourceService(sls, options);
                    return [4 /*yield*/, service.listDeployments()];
                case 1:
                    deploymentString = _b.sent();
                    expectedDeploymentString = "\n\nDeployments";
                    try {
                        for (deployments_2 = __values(deployments), deployments_2_1 = deployments_2.next(); !deployments_2_1.done; deployments_2_1 = deployments_2.next()) {
                            dep = deployments_2_1.value;
                            expectedDeploymentString += "\n-----------\n";
                            expectedDeploymentString += "Name: " + dep.name + "\n";
                            expectedDeploymentString += "Timestamp: None\n";
                            expectedDeploymentString += "Datetime: None\n";
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (deployments_2_1 && !deployments_2_1.done && (_a = deployments_2.return)) _a.call(deployments_2);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                    expectedDeploymentString += "-----------\n";
                    expect(deploymentString).toEqual(expectedDeploymentString);
                    return [2 /*return*/];
            }
        });
    }); });
    it("gets deployment template", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, resourceGroup, options, service, deploymentName, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    resourceGroup = "myResourceGroup";
                    sls.service.provider["resourceGroup"] = resourceGroup;
                    sls.variables["azureCredentials"] = "fake credentials";
                    options = mockFactory_1.MockFactory.createTestServerlessOptions();
                    service = new resourceService_1.ResourceService(sls, options);
                    deploymentName = "myDeployment";
                    return [4 /*yield*/, service.getDeploymentTemplate(deploymentName)];
                case 1:
                    result = _a.sent();
                    expect(arm_resources_1.ResourceManagementClient.prototype.deployments.exportTemplate)
                        .toBeCalledWith(resourceGroup, deploymentName);
                    expect(result).toEqual(template);
                    return [2 /*return*/];
            }
        });
    }); });
});
