"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var bindings_1 = require("./bindings");
var constants_1 = require("./constants");
var guard_1 = require("./guard");
var Utils = /** @class */ (function () {
    function Utils() {
    }
    Utils.getFunctionMetaData = function (functionName, serverless) {
        var bindings = [];
        var bindingSettingsNames = [];
        var bindingSettings = [];
        var bindingUserSettings = {};
        var bindingType;
        var functionsJson = { disabled: false, bindings: [] };
        var functionObject = serverless.service.getFunction(functionName);
        var handler = functionObject.handler;
        var events = functionObject["events"];
        var params = {
            functionJson: null
        };
        var parsedBindings = bindings_1.BindingUtils.getBindingsMetaData(serverless);
        var bindingTypes = parsedBindings.bindingTypes;
        var bindingDisplayNames = parsedBindings.bindingDisplayNames;
        for (var eventsIndex = 0; eventsIndex < events.length; eventsIndex++) {
            bindingType = Object.keys(functionObject["events"][eventsIndex])[0];
            if (eventsIndex === 0) {
                bindingType += constants_1.constants.trigger;
            }
            var index = bindingTypes.indexOf(bindingType);
            if (index < 0) {
                throw new Error("Binding  " + bindingType + " not supported");
            }
            serverless.cli.log("Building binding for function: " + functionName + " event: " + bindingType);
            bindingUserSettings = {};
            var azureSettings = events[eventsIndex][constants_1.constants.xAzureSettings];
            var bindingTypeIndex = bindingTypes.indexOf(bindingType);
            var bindingUserSettingsMetaData = bindings_1.BindingUtils.getBindingUserSettingsMetaData(azureSettings, bindingType, bindingTypeIndex, bindingDisplayNames);
            bindingTypeIndex = bindingUserSettingsMetaData.index;
            bindingUserSettings = bindingUserSettingsMetaData.userSettings;
            if (bindingType.includes(constants_1.constants.queue) && functionObject["events"][eventsIndex].queue) {
                bindingUserSettings[constants_1.constants.queueName] = functionObject["events"][eventsIndex].queue;
            }
            if (bindingTypeIndex < 0) {
                throw new Error("Binding not supported");
            }
            bindingSettings = parsedBindings.bindingSettings[bindingTypeIndex];
            bindingSettingsNames = parsedBindings.bindingSettingsNames[bindingTypeIndex];
            if (azureSettings) {
                for (var azureSettingKeyIndex = 0; azureSettingKeyIndex < Object.keys(azureSettings).length; azureSettingKeyIndex++) {
                    var key = Object.keys(azureSettings)[azureSettingKeyIndex];
                    if (bindingSettingsNames.indexOf(key) >= 0) {
                        bindingUserSettings[key] = azureSettings[key];
                    }
                }
            }
            bindings.push(bindings_1.BindingUtils.getBinding(bindingType, bindingSettings, bindingUserSettings));
        }
        if (bindingType === constants_1.constants.httpTrigger) {
            bindings.push(bindings_1.BindingUtils.getHttpOutBinding(bindingUserSettings));
        }
        functionsJson.bindings = bindings;
        params.functionsJson = functionsJson;
        var _a = Utils.getEntryPointAndHandlerPath(handler), handlerPath = _a.handlerPath, entryPoint = _a.entryPoint;
        if (functionObject["scriptFile"]) {
            handlerPath = functionObject["scriptFile"];
        }
        return {
            entryPoint: entryPoint,
            handlerPath: path_1.relative(functionName, handlerPath),
            params: params
        };
    };
    Utils.getEntryPointAndHandlerPath = function (handler) {
        var handlerPath = "handler.js";
        var entryPoint = handler;
        var handlerSplit = handler.split(".");
        if (handlerSplit.length > 1) {
            entryPoint = handlerSplit[handlerSplit.length - 1];
            handlerPath = handler.substring(0, handler.lastIndexOf(".")) + ".js";
        }
        var metaData = {
            entryPoint: entryPoint,
            handlerPath: handlerPath
        };
        return metaData;
    };
    Utils.prettyPrint = function (json) {
        return JSON.stringify(json, null, 2);
    };
    /**
     * Take the first `substringSize` characters from each string and return as one string
     * @param substringSize Size of substring to take from beginning of each string
     * @param args Strings to take substrings from
     */
    Utils.appendSubstrings = function (substringSize) {
        var e_1, _a;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var result = "";
        try {
            for (var args_1 = __values(args), args_1_1 = args_1.next(); !args_1_1.done; args_1_1 = args_1.next()) {
                var s = args_1_1.value;
                result += (s.substr(0, substringSize));
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (args_1_1 && !args_1_1.done && (_a = args_1.return)) _a.call(args_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    };
    /**
     * Creates a short name to be used for state name abbreviation
     * @param stageName The stage name
     */
    Utils.createShortStageName = function (stageName) {
        guard_1.Guard.empty(stageName);
        var stageMap = {
            "dogfood": "df",
            "production": "prod",
            "development": "dev",
            "testing": "test"
        };
        return this.createShortName(stageName, stageMap);
    };
    /**
     * Gets the normalized region name from long name (ex. West US 2 -> westus2)
     * @param regionName The region name
     */
    Utils.getNormalizedRegionName = function (regionName) {
        guard_1.Guard.empty(regionName);
        return regionName.replace(/\W/g, "").toLowerCase();
    };
    /**
     * Creates a short name for an azure region
     * @param regionName The azure region name
     */
    Utils.createShortAzureRegionName = function (regionName) {
        guard_1.Guard.empty(regionName);
        var locationMap = {
            "north": "n",
            "south": "s",
            "east": "e",
            "west": "w",
            "central": "c",
        };
        return this.createShortName(regionName, locationMap);
    };
    /**
     * Creates a short name from a long name based on a well-known string map
     * @param longName The long name to replace
     * @param wellKnownMap A well known map of long terms to short abbreviations
     */
    Utils.createShortName = function (longName, wellKnownMap) {
        guard_1.Guard.empty(longName);
        guard_1.Guard.null(wellKnownMap);
        var pattern = "(" + Object.keys(wellKnownMap).join("|") + ")";
        var regex = new RegExp(pattern, "g");
        return longName
            .replace(/\W+/g, "")
            .toLowerCase()
            .split(regex)
            .map(function (part) {
            return wellKnownMap[part] || part.substr(0, 3);
        })
            .join("");
    };
    Utils.get = function (object, key, defaultValue) {
        if (key in object) {
            return object[key];
        }
        return defaultValue;
    };
    Utils.getTimestampFromName = function (name) {
        var regex = /.*-t([0-9]+)/;
        var match = name.match(regex);
        if (!match || match.length < 2) {
            return null;
        }
        return match[1];
    };
    Utils.getIncomingBindingConfig = function (functionConfig) {
        return functionConfig.events.find(function (event) {
            var settings = event["x-azure-settings"];
            return settings && (!settings.direction || settings.direction === "in");
        });
    };
    Utils.getOutgoingBinding = function (functionConfig) {
        return functionConfig.events.find(function (event) {
            var settings = event["x-azure-settings"];
            return settings && settings.direction === "out";
        });
    };
    return Utils;
}());
exports.Utils = Utils;
